@@ -72,8 +72,8 @@ struct ne_ssl_pkcs11_provider_s {
 #include <openssl/rsa.h>
 #include <openssl/err.h>
 
-#if defined(RSA_F_RSA_OSSL_PRIVATE_ENCRYPT)
-#define PK11_RSA_ERR (RSA_F_RSA_OSSL_PRIVATE_ENCRYPT)
+#if defined(RSA_F_RSA_PRIVATE_ENCRYPT)
+#define PK11_RSA_ERR (RSA_F_RSA_PRIVATE_ENCRYPT)
 #else
 #define PK11_RSA_ERR (RSA_F_RSA_EAY_PRIVATE_ENCRYPT)
 #endif
@@ -113,13 +113,13 @@ static int pk11_rsa_encrypt(int mlen, const unsigned char *m,
         return 0;
     }
 
-    if (padding != RSA_PKCS1_PADDING && padding != RSA_NO_PADDING) {
+    if (padding != RSA_PKCS1_PADDING) {
         NE_DEBUG(NE_DBG_SSL, "pk11: Cannot sign, unknown padding mode '%d'.\n", padding);
         RSAerr(PK11_RSA_ERR,ERR_R_RSA_LIB);
         return 0;
     }        
 
-    mech.mechanism = padding == RSA_PKCS1_PADDING ? CKM_RSA_PKCS : CKM_RSA_X_509;
+    mech.mechanism = CKM_RSA_PKCS;
     mech.parameter = NULL;
     mech.parameter_len = 0;
 
